package edu.washington.cac.myuw.servlet.session;

import java.io.IOException;
import java.io.Serializable;
import java.util.Random;
import java.util.Date;

import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletRequest;

import edu.washington.cac.myuw.servlet.LastModifiedTimes;
import edu.washington.cac.myuw.servlet.Log;
import edu.washington.cac.myuw.servlet.LogFactory;
import edu.washington.cac.myuw.servlet.UWParameter;
import edu.washington.cac.myuw.servlet.ParameterException;

/** handles creating, deleting, verifying user sessions 

   @author Greg Barnes
   @version 2.0
 */

/* invariant for proving correctness:

  1. new HttpSessions are only created in start().  
  2. Every HttpSession a user has in UWSession is the same HttpSession 
    created the last time they called start().  
  3. No user has more than one HttpSession at a time unless he is
    on two different browser sessions.
*/

/* Note on use of synchronized below:

   With SessionManager, there is a concern about inconsistenty in
   UWSessions: one thread might want to use the underlying HttpSession 
   while another thread might want to delete it.

   To avoid this problem, SessionManager will synchronize on any UWSession
   before it invalidates the underlying HttpSession.

   Under this policy, it is difficult to avoid deadlock if the UWSession
   object has a lock on itself when calling SessionManager methods.  So
   we will insure we don't have the lock on ourselves when calling 
   SessionManager methods.  

   In most other instances, we synchronize all methods below (or major 
   parts of a method) where the session variable is accessed more than once,
   to avoid the mentioned inconsistency.  (A call to a method that 
   accesses the session counts as a single access).  

   Code in this class should not cause deadlock, as there is a partial 
   order on the methods based on which calls which.  

   Because session is private, no subclass can access it except 
   by calling the methods in this class.  Any subclass method that calls 
   more than one of the methods of this class should probably be synchronized 
   as well (except those that call the finalized methods and loadOldSession,
   because those methods call SessionManager methods).  
   The package-private getSession and setSession methods below 
   could be used to get around the control on the session variable; these
   methods should only be used by the SessionManager class.

   The arguments against deadlock in the SessionManager methods is 
   discussed in that class.

   Even with this synchronization, it's possible sessions will be 
   invalidated out from under us, so code that uses UWSession should be 
   prepared for InvalidSessions at any time.
 */

/* Note on use of final below:

   Methods that call SessionManager are finalized below (with the exception
   of loadOldSession, which is overridden by myuw.test.DummySession to do 
   nothing much).  This is done to insure that the SessionManager is made 
   aware of all creation, deletion, and access of HttpSessions.  Other 
   methods that are finalized can be unfinalized so they can be overriden as 
   well, but this should not be done without considering the demands of the 
   SessionManager.
 */
public class UWSession implements Serializable 
{
  protected HttpSession session = null;
  protected static final String USER_KEY = "user";  // used in the subclass
  protected static final String GUEST_KEY = "guest";
  protected static final String LOG_KEY = "mylog";
  protected boolean isGuest = false;

  /** An object to create logs */
  protected LogFactory logFactory;

  protected Log log = null;

  protected HttpServletRequest req;

  /** Create a new UWSession
    @param req The associated servlet request
    @param logFactory An object that can create a log
   */
  public UWSession(HttpServletRequest req, LogFactory logFactory) 
  {
    this.req = req;
    this.logFactory = logFactory;
    loadOldSession();
  }

  // Write the Session to an output stream
  private void writeObject(java.io.ObjectOutputStream out)
      throws IOException 
  {
    out.writeObject(session);
    out.writeObject(log);
  }

  private void readObject(java.io.ObjectInputStream in)
     throws IOException, ClassNotFoundException {}
  /**
    @return the session variable
   */
  /* 
    Classes other than SessionManager should avoid using this method.
    See the class comment on synchronization above
   */
  HttpSession getSession() 
  {
    return this.session;
  }

  /**
    Set the session variable
    @param session The new value of the session
   */
  /* 
    Classes other than SessionManager should avoid using this method.
    See the class comment on synchronization above
   */
  void setSession(HttpSession session) 
  {
    this.session = session;
  }

  /** perform other things after establishing a new session
      empty now, but subclasses can define
    @param Exception If there's a problem
    */
  protected void newSession(String uid) throws Exception
  {
  }

  /**
    Load the old <code>HttpSession</code>, if there is one
    @return The old HttpSession
   */
  /* 
    See the class comment on synchronized before overriding this method
   */
  protected void loadOldSession() 
  {
    SessionManager.loadOldSession(req, this);
  }

  /** flag to insure start is only called once */
  private boolean startCalled = false;

  /** Establish a user session.
      Delete any old session and create a new session.
    @param uid Name of the user
    @param log File to print messages
    @exception InvalidSession if the session becomes invalid
    @exception Exception if there's another problem
   */
  public final void start(String uid, Log log) throws InvalidSession, Exception
  {
    if (startCalled) {
      throw new IllegalStateException("UWSession::start called twice!");
    }

    startCalled = true;

    if (uid == null || log == null) {
      throw new IllegalArgumentException("UWSession::start null parameter.");
    }

    this.log = log;

    //Do not use isRequestedSessionIdValid()!!
    //After compilation, sometimes req.isRequestedSessionIdValid() 
    //return false even there is a old session.
    // Fang lin Mar 30, 2000
    try {
      //may throw NullPointerException
      SessionManager.createNewSession(req, this, 
                                      uid.equals(GUESTID) ?  log.getUid() : uid,
                                      uid.equals(GUESTID),
                                      log); 

      if (session != null) {
        session.setAttribute(USER_KEY, uid);
        session.setAttribute(LOG_KEY, log);

        if (isGuest) {
          // this need to be set before calling newSession; fanglin
          session.setAttribute(GUEST_KEY, GUESTID);
        }

        newSession(uid);
      }
    } catch (NullPointerException e ) {
      log.println("UWSession::start: " + e);
      session = null;
    }
  }

  public static final String GUESTID = "xyztseug";

  /** establish a guest session.
    @param log File to print messages to
    @exception InvalidSession if the session becomes invalid
    @exception Exception if there's a problem
   */
  public void startGuest(Log log) throws InvalidSession, Exception
  {
    isGuest = true;
    start(GUESTID, log);
  }

  /** invalidate this session 
   */
  public final void invalidate()
  {
    SessionManager.invalidate(this);
  }

  /** check if this user has a valid session.  

    @return The userid, if the session is valid
    @exception InvalidSession if the session is invalid.
   */
  public final String verify() throws InvalidSession 
  {
    if (session == null || session.isNew()) {
      SessionManager.invalidate(this);
      throw new InvalidSession("No current Session!");
    } else {
      /* we could synchronize the two lines below, but any inconsistency
         would just cause the verification to fail, which is not really
         worth trying to avoid in this method
       */
      getLog();
      String uid = getUserid();

      if (uid == null) {
        log.println("Failed on Verification (no userId)!");
        SessionManager.invalidate(this);
        throw new InvalidSession("Cannot find user name!");
      } else {
        return uid;
      }
    }
  }

  /** @return Does this session hold information about a particular user?
    @param userid The user.  If the user is a guest, this should be
      the unique id used in log messages, not GUEST_ID
   */
  boolean represents(String userid) {
    try {
      /* To represent a user, the log and the underlying session must
         agree on the userid (Note that in start(), the two may disagree
         if there is an old session, because we set the log before
         we create the new session).
       */
      return session != null && getLog().getUid().equals(userid) &&
        ((Log) getValue(LOG_KEY)).getUid().equals(userid);
    } catch (Exception e) {
      // 'getValue' had a problem
      return false;
    }
  }

  /** @return The user key represented by an <code>HttpSession</code>.  
      If the user is a guest, this is the unique id used in log messages, 
      not GUEST_ID
   */
  private static String sessionUserKey(HttpSession session) {
    try {
      return ((Log) session.getAttribute(LOG_KEY)).getUid();
    } catch (Exception e) {
      return null;
    }
  }

  /** @return The user represented by this session.  If the user is a guest,
       this is the unique id used in log messages, not GUEST_ID
   */
  String userKey() 
  {
    // Try to get from the underlying session first
    if (sessionUserKey(session) != null) {
      return sessionUserKey(session); 
    } else {
      // If there is no session, or there's a problem, try the log variable
      return (log == null) ? null : getLog().getUid();
    }
  }

      /* methods to mimic the dictionary-like data storage of HttpSession */
      /* Note that synchronization of putValue, getValue, etc. insures
         that the checks that were made in checkState methods hold true
         during calls to session getAttribute, putAttribute, etc. methods.
         In particular, this should mean the checkState methods will
         catch most IllegalStateExceptions that would have been thrown,
         and replace them with InvalidSessions instead.
       */
  /** 
     check that the state is okay before doing a get/invalidate
     @param key key name of object to get/invalidate
     @exception IllegalArgumentException if the <code>key</code> is null
     @exception InvalidSession if the session is null
   */
  private void checkState(String key) 
      throws IllegalArgumentException, InvalidSession
  {
    if (key == null) {
      throw new IllegalArgumentException("UWSession::checkState: key is null!");
    }

    if (session == null) {
      throw new InvalidSession("UWSession::checkState: session is null!");
    }
  }

  /** 
     check that the state is okay before doing a put
     @param key key name of object to put
     @param value object to put
     @exception IllegalArgumentException if the <code>key</code> or 
       <code>value</code>is null
     @exception InvalidSession if the session is null
   */
  private void checkState(String key, Object value) 
      throws IllegalArgumentException, InvalidSession
  {
    checkState(key);

    if (value == null) {
      throw new 
        IllegalArgumentException("UWSession::checkState: value is null!");
    }
  }

  /** associate a name/value pair with this session 
    @param name The name to which the data object will be bound. 
      This parameter cannot be null. 
    @param value The data object to be bound. This parameter cannot be 
      null. 
    @exception InvalidSession if the session is null or
                              if an attempt is made to access 
                              session data after the session has been invalidated, 
                              or before it has been initialized 
    */
  public synchronized void putValue(String key, Object value) 
    throws InvalidSession
  {
    checkState(key, value);
    try {
      session.setAttribute(key, value);
    } catch (IllegalStateException e) {
      throw new InvalidSession (e.getMessage());
    }
  }

  /** Returns the object bound to the given name in the session's data. 
      Returns null if there is no such binding. 

    @param name The name of the binding to find 
    @return the value bound to that name, or null if the binding does 
      not exist. 
    @exception InvalidSession if the session is null or
                              if an attempt is made to access 
                              HttpSession's session data after 
                              it has been invalidated, or before
                              it has been initialized
   */
  public synchronized Object getValue(String key) 
    throws InvalidSession
  {
    checkState(key);
    try {
      return session.getAttribute(key);
    } catch (IllegalStateException e) {
      throw new InvalidSession (e.getMessage());
    }
  }

  /** Removes the object bound to the given name in the session's data. 
    Does nothing if there is no object bound to the given name. 

    @param name The name of the object to remove 
    @exception InvalidSession if the session is null or
                              if an attempt is made to access 
                              session data after the session has been 
                              invalidated, or before it has been initialized
   */
  public synchronized void removeValue(String key) 
    throws InvalidSession
  {
    checkState(key);
    try {
      session.removeAttribute(key);
    } catch (IllegalStateException e) {
      throw new InvalidSession (e.getMessage());
    }
  }

  /** @return the userid for this session
      @exception InvalidSession if the session is null or otherwise bad
   */
  public String getUserid()
    throws InvalidSession
  {
    return (String) getValue(USER_KEY);
  }

  /** @return Whether the sesion belongs to a guest.  Returns false if the
      session is invalid
    */
  public boolean isGuest() {
    /* This method needs to work in the following 3 situations:
       before we have a session, after we have a session, after the
       session is invalidated.  

       Return true if the flag is set in this object, or if the 
       attribute is set for the session.  Return false if the session
       is invalid.
     */
    try {
      return (isGuest || GUESTID.equals((String) getValue(GUEST_KEY)));
    } catch (IllegalStateException e) {  
      // catches subclass InvalidSession as well
      return false;
    }
  }

  /**
    Create a new, userless, log.  
    @return the Log
   */
  protected Log createLog() 
  {
    return this.logFactory.create();
  }

  /** get the log for this user
    @return the log
    @exception InvalidSession if the session or its log is null
   */
  public synchronized Log getLog() throws InvalidSession 
  {
    if (log != null) {
      return log;
    }

    if (session == null) {
      log = createLog();
      throw new InvalidSession();
    } else {
      log = (Log) getValue(LOG_KEY);

      if (log == null || log.getPrintStream() == null) {
        log = createLog();
        throw new InvalidSession();
      }
    }

    return log;
  }

  /** label of a cache of information about the user */
  private static final String PROFILE = "profile";

  /** Save a profile in the Session cache
     @param profile Profile to save
   */
  public void setProfile(Profile profile)
  {
    putValue(PROFILE, profile);
  }

  /** Get the profile from the Session cache
     @return the Profile, or a new Profile if there is no such object
   */
  public Profile getProfile() {
    Object o = getValue(PROFILE);

    if (o == null) {
      return new Profile(this);
    } else if (o instanceof Profile) {
      Profile p = (Profile) o;
      p.setSession(this);
      return p;
    } else {
      // obviously, we've got a big problem now
      throw new IllegalStateException(
          "UWSession::getProfile: classloader failed");

/* Below was an attempt to fix the above problem, but I no longer 
   think this is a problem, so I will comment it out.  If
   we have to reimplement this, the method will probably need to be 
   synchronized. --- gsb 12/01
 */
//      synchronized (this) {
//        Enumeration names = session.getAttributeNames();
//
//        while (names.hasMoreElements()) {
//          String name = (String) names.nextElement();
//  
//          // remove everything that isn't the user's name
//          if (!name.equals(USER_KEY)) {
//            removeValue(name);
//          }
//        }
//      }
//
//      return new Profile(this);
    }
  }

  /** Name of the key used to keep track of the time pages were last modified */
  private static final String LAST_MODIFIED = "lastModified";

  /** get the last modified time for an id 
    @param id The id for which to get the last modified time
    @exception IllegalStateException if we can't find the set of
       last modified times
    */
  public synchronized long getLastModified(String id) 
      throws IllegalStateException 
  {
    if (session == null) {
       return LastModifiedTimes.ALREADY_EXPIRED;
    } else {
       LastModifiedTimes v = (LastModifiedTimes) getValue(LAST_MODIFIED);

       if (v==null) {
         throw new IllegalStateException(
             "session lost lastModified entry! aborting...");
       } else {
         return v.getLastModified(id);
       }
    }
  }

  /** set the last modified time for an id to right now
    @param id The id for which to set the last modified time
      If id is null, <em>all</em> last modified times are set 
      to right now.
    */
  public void setLastModified(String id) 
  {
    setLastModified(id, -1);
  }

  /** set the last modified time for an id to right now, with an expiration
    @param id The id for which to set the last modified time
      If id is null, <em>all</em> last modified times are set 
      to right now.
    @param expire milliseconds until we assume the page has expired
      If expire is negative, no expiration time(s) are set
      If expire is not negative, and id is null, expiration times
        are set for all last modified times that currently have
        an expiration already
   */
  public synchronized void setLastModified(String id, long expire) 
  {
    LastModifiedTimes v = (LastModifiedTimes) getValue(LAST_MODIFIED);

    if (v == null && id == null) {
      return;
    }

    if (v == null) {
      v = new LastModifiedTimes();
    }

      // see logic above in javadoc
    if (id !=null && expire < 0) {
      v.setLastModified(id);
    } else if (id !=null && expire >= 0) {
      v.setLastModified(id, expire);
    } else if (id ==null && expire >= 0) {
      v.setLastModified(expire);
    } else {
      v.setLastModified();
    }

    putValue(LAST_MODIFIED,v);
  }

  protected static final String ID_KEY = "sessid"; 

  /**Return the unique identifier of the Http session.
     Each session must have its own pseudorandom-number generator.
  */
  public int getIdentifier () 
  {
    Integer id = (Integer) getValue(ID_KEY);
    if (id == null) {
      id = new Integer( new Random().nextInt() );
      putValue ( ID_KEY, id );
    }
    return id.intValue();
  }

  /** @return true if the identifier value matches with
      the value stored in the httpSeesion */
  public boolean matchedIdentifier (int value) 
  {
    return ( getValue(ID_KEY) != null &&
	     ((Integer)getValue(ID_KEY)).intValue() == value );
  }

  
}
