// java

package uwcal;

import java.io.IOException;
import java.sql.SQLException;

import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.CaldataWarning;
import edu.washington.cac.calendar.data.DuplicateItemException;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.ItemException;
import edu.washington.cac.calendar.data.NoSuchItemException;
import edu.washington.cac.calendar.data.SubscriptionItemException;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.myuw.servlet.Log;
import edu.washington.cac.myuw.util.Coder;

import uwcal.out.DisplayText;
import uwcal.out.Page;

/**
  The action taken by the UWCal servlet.
  @author Greg Barnes
  @version 1.1
 */
public abstract class CalAction 
{
  /** The 'current' date */
  protected MyCalendar cal;

  /** The user performing the action */
  protected User user;

  /** The user's events cache */
  protected EventsI es;

  /** A message to print at the top of the page */
  protected String message = null;

  /** The form parameters, if any, from the previous page */
  protected CalParameter cp;

  /** A File to print message to */
  private Log log;

  /** The name of the function this action handles */
  private String handlesFunction;

  /** @return a new CalAction
    @param cal The date in question
    @param user The user performing the action
    @param es The events for the user
    @param cp Form parameters
    @param log File to print messages to
   */
  abstract CalAction newInstance(MyCalendar cal, 
                                 User user, 
                                 EventsI es, 
                                 CalParameter cp, 
                                 Log log);

  protected abstract void basePerformAction() 
      throws SQLException, ItemException, CaldataException;
  protected abstract void baseGeneratePage(HttpServletResponse resp) 
      throws IOException, ItemException, SQLException,
             CaldataException;

 /**
    Create a minimal CalAction, not suitable for performing actions.
    This object can be used to match function names with the proper 
    CalAction class, and calling newInstance to create a more complete 
    version of the object which can perform the action

    @param function Name of the function this CalAction can handle
   */
  CalAction(String handlesFunction) {
    this.handlesFunction = handlesFunction;
  }

  /** Create a CalAction to perform an action

    @param function The function that was called
    @param cal The day for the action
    @param user The user performing the action
    @param es The cache of events for the user
    @param cp The form parameters
    @param log A file to print messages to
   */
  protected CalAction(String function,         
                      MyCalendar cal, 
                      User user, 
                      EventsI es,
                      CalParameter cp,
                      Log log) 
  {
    this(function);
    this.cal = cal;
    this.user = user;
    this.es = es;
    this.cp = cp;
    this.log = log;
  }

  /** @return The name of the function this action handles */
  protected String getHandlesFunction() 
  {
    return this.handlesFunction;
  }

  /** @return Whether this action handles a particular function 
    @param function The function name to test
    */
  public boolean handles(String function) {
    return this.handlesFunction.equals(function);
  }

  /** @return The file to print messages to */
  protected Log getLog() 
  {
    return this.log;
  }

  /** Perform the action associated with this CalAction, not including
      page generation
   */
  void performAction() {
    try {
      basePerformAction();
    } catch (SQLException e) {
      e.printStackTrace(this.log.getPrintStream());
      this.message = DisplayText.addMessage(this.message, 
          DisplayText.genericErrorMessage(this.handlesFunction) + ": " +
          DisplayText.DB_ERROR);
    } catch (DuplicateItemException e) {
      this.message = DisplayText.addMessage(this.message, 
          DisplayText.genericErrorMessage(this.handlesFunction) + ": " +
          DisplayText.DUPLICATE_ITEM);
    } catch (SubscriptionItemException e) {
      this.message = DisplayText.addMessage(this.message, 
          DisplayText.genericErrorMessage(this.handlesFunction) + ": " +
          DisplayText.SUBSCRIPTION_ITEM);
    } catch (NoSuchItemException e) {
      this.message = DisplayText.addMessage(this.message, 
          DisplayText.genericErrorMessage(this.handlesFunction) + ": " +
          DisplayText.NO_SUCH_ITEM);
    } catch (CaldataWarning e) {
      this.message = DisplayText.addMessage(this.message, 
          "Some fields changed: " + Coder.HTMLencode(e.getMessage()));
    } catch (CaldataException e) {
      this.message = DisplayText.addMessage(this.message, 
          DisplayText.genericErrorMessage(this.handlesFunction) + ": " +
          Coder.HTMLencode(e.getMessage()));
    } catch (ItemException e) {
      this.message = DisplayText.addMessage(this.message, 
          DisplayText.genericErrorMessage(this.handlesFunction) + ": " +
          Coder.HTMLencode(e.getMessage()));
    }
  }

  /**
    print the page to the servlet's output stream

    @param resp The servlet's response object
    @exception IOException if there's a problem printing to the output stream
    @exception SQLException if there's a problem getting data from
      the database
    @exception CaldataException if the database contains bad data
   */
  void generatePage(HttpServletResponse resp) 
    throws IOException, SQLException, CaldataException
  {
    try {
      baseGeneratePage(resp);
      return;
    } catch (NoSuchItemException e) {
      message = DisplayText.addMessage(message, 
        DisplayText.genericErrorMessage(this.handlesFunction) + ": " +
          DisplayText.NO_SUCH_ITEM);
    } catch (CaldataException e) {
      message = DisplayText.addMessage(message, 
        DisplayText.genericErrorMessage(this.handlesFunction) + ": " +
          Coder.HTMLencode(e.getMessage()));
    } catch (ItemException e) {
      message = DisplayText.addMessage(message, 
          DisplayText.genericErrorMessage(this.handlesFunction) + ": " +
          Coder.HTMLencode(e.getMessage()));
    }

    try {
      // when all else fails, show the default page
      Page.create(CalParameter.DEFAULT_VIEW, cal, this.user, 
                  es, message, cp).print(resp);
    } catch (ItemException e) {
      throw new RuntimeException(
        "creating dayview shouldn't cause ItemException, but it did");
    }
  }
}
