// java

package uwcal;

import java.io.IOException;
import java.sql.SQLException;
import java.util.TreeSet;

import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.DuplicateItemException;
import edu.washington.cac.calendar.data.EntityRef;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.EventRef;
import edu.washington.cac.calendar.data.ItemException;
import edu.washington.cac.calendar.data.PersonalEvents;
import edu.washington.cac.calendar.data.PublicEvents;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.db.Caldata;
import edu.washington.cac.calendar.filter.CalendarRef;
import edu.washington.cac.myuw.servlet.Log;

import uwcal.out.DisplayText;
import uwcal.out.Page;

/**
  Actions in the calendar application that acts on references to other
  objects (currently, events or calendars)

  @author Greg Barnes
  @version 1.0
 */
public class RefAction extends DBAction 
{
  /** The object the action is performed on, if there is only one */
  private EntityRef er = null;  

  /** The set of objects the action is performed on, if there are more 
      than one */
  private TreeSet ers = new TreeSet();

  /**
    Create a minimal RefAction, not suitable for performing actions.
    This object can be used to match function names with the proper 
    RefAction class, and calling newInstance to create a more complete 
    version of the object which can perform the action

    @param function Name of the function this RefAction can handle
    @param successMessage message to be displayed if the action succeeds
   */
  RefAction(String function, String successMessage) 
  {
    super(function, successMessage);
  }

  /** Create a new RefAction to perform actions

    @param function The function that was called
    @param cal The day for the action
    @param user The user performing the action
    @param es The EventsI object for the user
    @param cp The form parameters
    @param log A file to print messages to
    @param successMessage Message to display if the action succeeds
   */
  protected RefAction(String function,       
                      MyCalendar cal,
                      User user, 
                      EventsI es,
                      CalParameter cp,
                      Log log,
                      String successMessage)
  {
    super(function, cal, user, es, cp, log, successMessage);
  }

  /** Create a new RefAction
    @param cal The day for the action
    @param user The user performing the action
    @param es The EventsI object for the user
    @param cp The form parameters
    @param log A file to print messages to
   */
  CalAction newInstance(MyCalendar cal, 
                        User user, 
                        EventsI es, 
                        CalParameter cp,
                        Log log) 
  {
    return new RefAction(getHandlesFunction(), cal, user, es, cp, log, 
                              getSuccessMessage());
  }

  /**
    Store all the calendars to subscribe to
    @param ids Ids of the calendars
   */
  protected void storeSubscriptions(int[] ids)
  {
    if (ids == null) {
      return;
    }

    for (int i = 0; i < ids.length; i++) {
      this.ers.add(new Integer(ids[i]));
    }
  }

  /** Set up the variables needed to perform the action
    @exception CaldataException If there's a problem with the form data
    @exception SQLException If there's a problem with the database
   */
  protected void setVars()
  {
    if (getHandlesFunction().endsWith(CalParameter.EVENTREF)) {
      this.er = new EventRef(cp.getEventID(), this.user);
    } else if (getHandlesFunction().endsWith(CalParameter.CALENDAR)) {
      storeSubscriptions(cp.getCalendars());
    } else {
      throw new IllegalStateException("setVars: illegalAction " + 
                                      getHandlesFunction());
    }
  }

  /** Start of an SQL state that indicates an integrity contraint was
     violated.
   */
  /* Note:  I cannot find the X/Open SQL spec on which these codes
     are based; I base this seemingly arbitrary number on an IBM
     list of SQLstate error codes, which you can find at:
     http://www.redbooks.ibm.com/tstudio/db2_400/CLI/sqlmsg/allhtml.htm

     Note that other lists of codes I could find (i.e., Informix's 
     cursory list) at least agree that if it starts
     with '23', it's an 'integrity constraint violation'.
   */
  private static final String INTEGRITY_PREFIX = "23";

  /**
    @return Is the given exception one that indicates an integrity 
    constraint was violated?
    @param e The exception in question
   */
  private boolean isConstraintException(SQLException e) 
  {
    return e.getSQLState() != null &&
           e.getSQLState().startsWith(INTEGRITY_PREFIX);
  }

  /**
    Perform the portion of the action that accesses the database

    @exception SQLException If there's a problem with the database
    @exception ItemException if there's a problem with the item
   */
  protected void perfAction() 
      throws SQLException, ItemException
  {
    if (getHandlesFunction().endsWith(CalParameter.CALENDAR)) {
      int[] results = ((PersonalEvents) this.es).fixSubscriptions(this.ers);
      setSuccessMessage(DisplayText.subscribeMessage(results));
    } else if (getHandlesFunction().startsWith(CalParameter.ADD)) {
      // must be event
      try {
        // right now, can only act on one Event at a time
        ((PersonalEvents) this.es).addRef(this.er, this.cal);
      } catch (SQLException e) {
        if (isConstraintException(e)) {
          // seems likely this was caused by duplicating an item
          throw new DuplicateItemException(e.getMessage());
        } else {
          throw e;
        }
      }
    } else if (getHandlesFunction().startsWith(CalParameter.DELETE) || 
               getHandlesFunction().startsWith(CalParameter.RDELETE)) 
    {
      ((PersonalEvents) this.es).deleteRef(this.er);
    } else {
      throw new IllegalStateException("perfAction: illegalAction " + 
        getHandlesFunction());
    }
  }
}
